#include <stdio.h>

#include "led_monitor.h"
#include "uart.h"
#include "i2c_master.h"

// i2c address
#define		I2C_ADR			0xA4		// ( 0x52 << 1 )


volatile int timer_val;
void wait(int tmax)
{
	int t; for (t=0;t<tmax;t++) { timer_val=t; }
}

int main()
{
	int i;
	int r;
	unsigned char data_buf[6];
	char buf[255];
	int gyro1,gyro2,gyro3;

	i2c_master_init();
	led_monitor_init(); // monitor LED setting
	
	rs_init(921600); // UART initialize

	rs_puts("\n");
	rs_puts("Wii Motion Plus test\n");
	rs_puts("\n");
	
	led_monitor_on_off(1); // indicator LED on
	
    // Wii Motion Plus initialize
	
	r=i2c_start( 0xA6 ,I2C_WRITE_MODE);  // 0xA6 == ( 0x52 << 1 )
	i2c_write( 0xFE ); i2c_write( 0x04 );
	i2c_stop();
	if (r!=0) { rs_puts("Error: no ACK\n"); }
	
	wait(500);
	
	led_monitor_on_off(0); // monitor LED off


	while (1) {
		
		led_monitor_on_off(1); // indicator LED on
		
		while (1) {
			r=i2c_start(I2C_ADR,I2C_WRITE_MODE);
			i2c_write( 0x00 );
			i2c_stop();
			if (r!=0) {
				// rs_puts("Error: no ACK\n");
				continue;
			}
		
			r=i2c_start(I2C_ADR,I2C_READ_MODE);
			for (i=0;i<6;i++) { data_buf[i]=0; }
			for (i=0;i<6;i++) {
				if (i!=5) {
					data_buf[i] = i2c_read(I2C_READ_WITH_ACK);
				} else {
					data_buf[i] = i2c_read(I2C_READ_WITH_NACK);
				}
			}
			i2c_stop();
			if (r!=0) {
				// rs_puts("Error: no ACK (2)\n")
				continue;
			} else {
				break;
			}
		}
		
		led_monitor_on_off(0); // LED off	
		
		gyro1 = data_buf[0]+(data_buf[3]>>2)*256; // yaw
		gyro2 = data_buf[1]+(data_buf[4]>>2)*256; // pitch
		gyro3 = data_buf[2]+(data_buf[5]>>2)*256; // roll
		
		sprintf(buf,"%d," ,gyro1); rs_puts(buf);
		sprintf(buf,"%d," ,gyro2); rs_puts(buf);
		sprintf(buf,"%d\n",gyro3); rs_puts(buf);
		
		wait(2000);
	}
	
	led_monitor_on_off(0); // LED off	
	return 0;
}
